unit IFEng;
interface
uses
  Classes;

type
  GLPropertyList = class(TStringList);

  GLPropertyType = (Bol, Int, Dbl, Str, Lst, Misc);
  GLProperty = class(TObject)
    Typ:        GLPropertyType;  { property type }
    Nam:        string[31];      { property name }
    Val:        string;          { property value string }
    View:       boolean;         { viewable in object inspector }
    Write:      boolean;         { writeable via object inspector }
  public
    constructor CreateAs(aTyp: GLPropertyType;
                         aNam, aVal: string;
                         aView, aWrite: boolean);
    function    GetType: GLPropertyType;
    function    GetName: string;
    function    GetValue: string;
    function    IsViewable: boolean;
    function    IsWriteable: boolean;
    function    GetBolValue: boolean;
    function    GetIntValue: integer;
    function    GetDblValue: double;
    function    GetStrValue: string;
    function    GetLstValue: GLPropertyList;
    function    GetMiscValue: string;
    function    Clone: GLProperty;
  end;

  GLElement = class(TObject)
  protected
    _props:     GLPropertyList;
    constructor Create;
    destructor  Destroy; override;
  public
    procedure   AddProperty(prop: GLProperty);
    function    GetVerbName(idx: integer): string; virtual;
    procedure   ExecuteVerb(idx: integer); virtual;
    function    GetPropertyCount: integer;
    function    GetPropertyByName(aNam: string): GLProperty;
    function    SetPropertyByName(aNam: string; val: string): boolean;
    function    GetPropertyByIndex(idx: integer): GLProperty; virtual;
    function    SetPropertyByIndex(idx: integer; aVal: string): boolean; virtual;
    function    Clone: GLElement; virtual;
  end;

  GLElementBuilder = class(TObject)
  public
    procedure   Open; virtual; abstract;
    procedure   Close; virtual; abstract;
    function    GetElementFirst: GLElement; virtual; abstract;
    function    GetElementNext: GLElement; virtual; abstract;
  end;

implementation
uses
  SysUtils;

{===================   GLProperty   =====================}

constructor GLProperty.CreateAs(aTyp: GLPropertyType;
                                aNam, aVal: string;
                                aView, aWrite: boolean);
begin
  Typ := aTyp;
  Nam := aNam;
  Val := aVal;
  View := aView;
  Write := aWrite;
end;

function GLProperty.GetType: GLPropertyType;
begin  result := Typ;  end;

function GLProperty.GetName: string;
begin  result := Nam;  end;

function GLProperty.GetValue: string;
begin  result := Val;  end;

function GLProperty.IsViewable: boolean;
begin  result := View;  end;

function GLProperty.IsWriteable: boolean;
begin  result := Write;  end;

function GLProperty.GetBolValue: boolean;
begin
  if (CompareText(val, 'true')=0) then result := true
  else                                 result := false;
end;

function GLProperty.GetIntValue: integer;
begin  result := StrToIntDef(Val, 0);  end;

function GLProperty.GetDblValue: double;
begin
  try
    result := StrToFloat(Val);
  except on EConvertError do
    result := 0.0;
  end;
end;

function GLProperty.GetStrValue: string;
begin  result := Val;  end;

function GLProperty.GetLstValue: GLPropertyList;
begin  result := nil;  end;

function GLProperty.GetMiscValue: string;
begin  result := Val;  end;

function GLProperty.Clone: GLProperty;
begin
  result := GLProperty.Create;
  result.Typ := Typ;
  result.Nam := Nam;
  result.Val := Val;
  result.View := View;
  result.Write := Write;
end;

{===================      GLElement      =====================}
constructor GLElement.Create;
begin  _props := GLPropertyList.Create;  end;

destructor GLElement.Destroy;
begin  _props.Free;  end;

procedure GLElement.AddProperty(prop: GLProperty);
begin  _props.AddObject(prop.Nam, prop);  end;

function GLElement.GetVerbName(idx: integer): string;
begin  result := '';  end;

procedure GLElement.ExecuteVerb(idx: integer);
begin  { empty  }  end;

function GLElement.GetPropertyCount: integer;
begin  result := _props.Count;  end;

function GLElement.GetPropertyByName(aNam: string): GLProperty;
var  i: integer;
begin
  i := _props.IndexOf(aNam);
  if (i<0) then result := nil
  else          result := GetPropertyByIndex(i);
end;

function GLElement.SetPropertyByName(aNam: string; val: string): boolean;
var i: integer;
begin
  i := _props.IndexOf(aNam);
  if (i<0) then result := false
  else          result := SetPropertyByIndex(i, val);
end;

function GLElement.GetPropertyByIndex(idx: integer): GLProperty;
begin
  if (idx>=0) and (idx<_props.Count) then
    result := GLProperty(_props.Objects[idx])
  else
    result := nil;
end;

function GLElement.SetPropertyByIndex(idx: integer; aVal: string): boolean;
begin
  if (idx<0) or (idx>=_props.Count) then
    result := false
  else begin
    with _props.Objects[idx] as GLProperty do
      Val := aVal;
    result := true;
  end;
end;

function GLElement.Clone: GLElement;
var i: integer;
begin
  result := GLElement.Create;
  i := 0;
  while (i < _props.Count) do begin
    result.AddProperty(GetPropertyByIndex(i).Clone);
    Inc(i);
  end;
end;

end.

